#include "config.h"
#include "loadfile.h"
#include "expload.h"
#include "cdbslib.h"

/* USE_RULES -- Apply the rules to expand the input table
 *
 * i: itab	input table descriptor
 * o: otab	output (expanded) table descriptor
 * i: target	target table descriptor
 * i: action 	action table descriptor
 */

void use_rules (table *itab, table *otab, table *target, table *action)
{
    table	*work;
    int		first, iwork, irow, orow;

    /* Create the work table to hold intermediate results */

    first = YES;
    work = create_table ();
    copy_layout (itab, work, "_TARGET,_INDEX,_USED");

    /* Loop over each row in the input table */

    orow = 0;
    for (irow = 0; irow < nrow_table(itab); irow ++) {
	/* Copy the row to the work table and compute 
	 * all possible expansions
	 */

        if (first == NO) {
	    iwork = nrow_table(work);

	} else {
	  /* The first pass needs to be treated as a special case
	   * because creating the work table makes a table with
	   * a single, empty first row (row zero)
	   */
	    iwork = 0;
	    first = NO;
	}

        move_tabrow (itab, irow, work, iwork);

	for ( ; ; ) {
	    /* Find expansion for top row of work table
	     * Push expansion onto work table as if a stack
	     * Loop until no more expansions for top row
	     */

	    while (find_rule (target, work) != ERR)
	        apply_rule (target, action, work);

	    iwork = nrow_table(work) - 1;
	    if (iwork < 0)
	        break;

	    /* Copy fully expanded row to output table */

	    move_tabrow (work, iwork, otab, orow);
	    put_tab_cell (otab, "EXPANSION_NUMBER", orow, itoa(orow+1));
	    trunc_table (work, iwork);
	    orow ++;
	}
    }

    free_table (work);
}
